@@ -1,4 +1,4 @@
-/*
+/* vi: set expandtab sw=2 sts=2:
  * sato-draw.c
  * This file is part of sato-engine
  *
@@ -20,6 +20,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
+#ifndef ENABLE_CAIRO
 
 #include "sato-draw.h"
 #include "sato-style.h"
@@ -158,73 +161,52 @@ sato_rounded_rectangle (GdkWindow * window, GdkGC * gc, gint x, gint y,
 
 }
 
-
-static void
-sato_gradient (GtkStyle * style, GdkWindow * window, GtkStateType state_type,
-	       gint x, gint y, gint width, gint height)
-{
-  gint i, rd, gd, bd;		/* rd, gd, bd - change in r g and b for gradient */
-  GdkColor start_color, end_color;
-  GdkGC *gc;
-  gc = gdk_gc_new (window);
-
-  /* get the start and end colours */
-  if (state_type == GTK_STATE_ACTIVE)
-  {
-    start_color = style->bg[state_type];
-    sato_shade_colour (&style->bg[state_type], &end_color, 1.15);
-  }
-  else
-  {
-    sato_shade_colour (&style->bg[state_type], &start_color, 1.15);
-    end_color = style->bg[state_type];
-  }
-
-  /* set line for 1px */
-  gdk_gc_set_line_attributes (gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
-			      GDK_JOIN_MITER);
-
-
-  /* calculate the delta values */
-
-  rd = (start_color.red - end_color.red) / MAX (height, 1);
-  gd = (start_color.green - end_color.green) / MAX (height, 1);
-  bd = (start_color.blue - end_color.blue) / MAX (height, 1);
-
-  i = -1;
-  while (++i < height)
-  {
-    gdk_gc_set_rgb_fg_color (gc, &start_color);
-    gdk_draw_line (window, gc, x, y + i, x + width, y + i);
-    start_color.red -= rd;
-    start_color.blue -= bd;
-    start_color.green -= gd;
-  }
-
-  /* round off the gradient corners */
-
-  gdk_draw_point (window, style->bg_gc[state_type], x, y);
-  gdk_draw_point (window, style->bg_gc[state_type], x, y + height - 1);
-  gdk_draw_point (window, style->bg_gc[state_type], x + width - 1,
-		  y + height - 1);
-  gdk_draw_point (window, style->bg_gc[state_type], x + width - 1, y);
-
-}
-
 static void
 sato_draw_box (DRAW_ARGS)
 {
-  GdkGC *gc;
   gboolean has_focus;
+  GdkGC *gc;
+  GdkGC *border_color = style->fg_gc[state_type];
 
   DEBUG ("draw_box");
-  SANITIZE_SIZE;
 
-  if (DETAIL ("paned") || DETAIL ("menubar") || DETAIL ("toolbar"))
+  if (DETAIL ("paned") || DETAIL ("vscrollbar") || DETAIL ("hscrollbar"))
     return;
 
+  /* hack to remove PRELIGHT state */
+  if (!DETAIL ("menuitem") && !DETAIL ("bar") && state_type == GTK_STATE_PRELIGHT)
+  {
+    if (widget && GTK_IS_TOGGLE_BUTTON (widget) && shadow_type == GTK_SHADOW_IN)
+    {
+      state_type = GTK_STATE_ACTIVE;
+    }
+    else
+    {
+      if (widget && GTK_IS_BUTTON (widget)
+          && (gtk_button_get_relief (GTK_BUTTON (widget)) == GTK_RELIEF_NONE))
+      {
+        /* none relief buttons shouldn't draw anything for "normal" state */
+        return;
+      }
+      else
+      {
+        state_type == GTK_STATE_NORMAL;
+      }
+    }
+  }
+
+  SANITIZE_SIZE;
+
+
+  /*** treeview headers ***/
+  if (widget && GTK_IS_TREE_VIEW (widget->parent))
+  {
+    return;
+  }
 
   has_focus = (widget && GTK_WIDGET_HAS_FOCUS (widget));
+  if (has_focus && state_type != GTK_STATE_ACTIVE)
+      border_color = style->bg_gc[GTK_STATE_SELECTED];
 
   /*** spin buttons ***/
   if (DETAIL ("spinbutton_down") || DETAIL ("spinbutton_up"))
@@ -245,66 +227,38 @@ sato_draw_box (DRAW_ARGS)
     entry = g_object_get_data (G_OBJECT (widget->parent), "sato-combo-entry");
     if (GTK_IS_ENTRY (entry))
     {
-      gtk_widget_queue_draw_area (entry, entry->allocation.x, entry->allocation.y, entry->allocation.width,entry->allocation.height);
+      gtk_widget_queue_draw_area (entry, entry->allocation.x, entry->allocation.y,
+                                  entry->allocation.width,entry->allocation.height);
 
-      if (!has_focus)
-        has_focus = GTK_WIDGET_HAS_FOCUS (entry);
+      has_focus = GTK_WIDGET_HAS_FOCUS (entry);
     }
 
     g_object_set_data (G_OBJECT (widget->parent), "sato-combo-button", widget);
+    if (has_focus)
+      border_color = style->bg_gc[GTK_STATE_SELECTED];
+    else
+      border_color = style->fg_gc[state_type];
 
     /* FIXME: RTL */
     width += 10;
     x -= 10;
 
-
   }
 
 
-  if (has_focus)
-    gc = sato_gc_new (style->bg_gc [GTK_STATE_SELECTED], window);
+  gc = sato_gc_new (border_color, window);
+
+  /* menu and toolbars get just a single line at the bottom of the widget */
+  if (DETAIL ("menubar") || DETAIL ("toolbar"))
+  {
+    gdk_draw_line (window, gc, x, y + height - 1, x + width, y + height - 1);
+  }
   else
-    gc = sato_gc_new (style->text_gc[state_type], window);
-
-
-  /*** draw the gradient ***/
-
-  if (!DETAIL ("menu") && !DETAIL ("trough"))
-   sato_gradient (style, window, state_type, x+1, y+1, width-2, height-2);
-
-
-
-   /*** treeview headers ***/
-  if (widget && GTK_IS_TREE_VIEW (widget->parent))
-  {
-    goto exit;
-  }
-
-
-  if (DETAIL ("trough"))
-  {
-    if (widget && GTK_IS_HSCALE (widget))
-    {
-      gdk_draw_line (window, gc, x, y + height / 2, x + width, y + height / 2);
-      goto exit;
-    }
-    else if (widget && GTK_IS_VSCALE (widget))
-    {
-      gdk_draw_line (window, gc, x + width / 2, y, x + width / 2, y + height);
-      goto exit;
-    }
-    else
-      gdk_draw_rectangle (window, style->base_gc[state_type], TRUE, x, y, width, height);
-
-  }
-
-  /*** draw the border ***/
-  if (!DETAIL ("bar"))
   {
+    gdk_draw_rectangle (window, style->bg_gc[state_type], TRUE, x + 1, y + 1, width - 2, height - 2);
     sato_rounded_rectangle (window, gc, x, y, width, height);
   }
 
-exit:
   g_object_unref (gc);
 
 }
@@ -332,14 +286,17 @@ sato_draw_shadow (DRAW_ARGS)
 
     button = g_object_get_data (G_OBJECT (widget->parent), "sato-combo-button");
     if (GTK_IS_BUTTON (button))
-      gtk_widget_queue_draw_area (button, button->allocation.x, button->allocation.y, button->allocation.width,button->allocation.height);
+      gtk_widget_queue_draw_area (button, button->allocation.x, button->allocation.y,
+                                  button->allocation.width,button->allocation.height);
   }
 
+  /*** draw the border ***/
+
   /* draw a hilight shadow on focused widgets (i.e. entry widgets) */
   if (widget && GTK_WIDGET_HAS_FOCUS (widget))
-    gc = sato_gc_new (style->base_gc[GTK_STATE_SELECTED], window);
+    gc = sato_gc_new (style->bg_gc[GTK_STATE_SELECTED], window);
   else
-    gc = sato_gc_new (style->text_gc[state_type], window);
+    gc = sato_gc_new (style->fg_gc[state_type], window);
 
   sato_rounded_rectangle (window, gc, x, y, width, height);
 
@@ -351,12 +308,8 @@ sato_draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
                  GdkRectangle *area, GtkWidget *widget, const gchar *detail,
                  gint x, gint y, gint width, gint height)
 {
-  GdkGC *gc;
   DEBUG ("draw_focus");
-
-  gc = sato_gc_new (style->fg_gc[state_type], window);
-  sato_rounded_rectangle (window, gc, x, y, width, height);
-  g_object_unref (gc);
+  /* TODO */
 }
 
 static void
@@ -369,21 +322,24 @@ sato_draw_check (GtkStyle * style, GdkWindow * window,
   GdkGC *gc;
 
   DEBUG ("draw_check");
-  gc = sato_gc_new (style->text_gc[state_type], window);
 
-  /* clear the background */
-  gdk_draw_rectangle (window, style->base_gc[GTK_STATE_NORMAL], TRUE, x+1, y+1, width-2, height-2);
 
+  /* fill the background */
+  if (widget && GTK_WIDGET_HAS_FOCUS (widget) && !GTK_IS_TREE_VIEW (widget))
+    gdk_draw_rectangle (window, style->base_gc[GTK_STATE_SELECTED], TRUE, x, y, width, height);
+  else
+    gdk_draw_rectangle (window, style->base_gc[state_type], TRUE, x, y, width, height);
+
+  /* draw the border */
+  gc = sato_gc_new (style->fg_gc[state_type], window);
+  gdk_draw_rectangle (window, gc, FALSE, x, y, width-1, height-1);
+
+  /*** draw check mark ***/
   if (shadow_type == GTK_SHADOW_IN)
   {
-    GdkGC *mark_gc;
-    mark_gc = sato_gc_new (style->base_gc[GTK_STATE_SELECTED], window);
-    gdk_draw_rectangle (window, mark_gc, TRUE, x + 5, y + 5, width - 10, height - 10);
-    g_object_unref (mark_gc);
+    gdk_draw_rectangle (window, style->text_gc[state_type], TRUE, x + 3, y + 3, width - 7, height - 7);
   }
 
-  sato_rounded_rectangle (window, gc, x, y, width, height);
-
   g_object_unref (gc);
 }
 
@@ -398,21 +354,21 @@ sato_draw_option (GtkStyle * style, GdkWindow * window,
 
   DEBUG ("draw_option");
 
-  gc = sato_gc_new (style->text_gc[state_type], window);
+  /* fill the background */
+  if (widget && GTK_WIDGET_HAS_FOCUS (widget) && !GTK_IS_TREE_VIEW (widget))
+    gdk_draw_arc (window, style->base_gc[GTK_STATE_SELECTED], TRUE, x+1, y+1, width-2, height-2, 0, 360 * 64);
+  else
+    gdk_draw_arc (window, style->base_gc[state_type], TRUE, x+1, y+1, width-2, height-2, 0, 360 * 64);
 
+  /* draw the border */
+  gc = sato_gc_new (style->fg_gc[state_type], window);
+  gdk_draw_arc (window, gc, FALSE, x, y, width - 1, height - 1, 0, 360 * 64);
 
-  /* clear the background */
-  gdk_draw_arc (window, style->base_gc[GTK_STATE_NORMAL], TRUE, x+1, y+1, width-2, height-2, 0, 360 * 64);
-
+  /*** draw check mark ***/
   if (shadow_type == GTK_SHADOW_IN)
   {
-    GdkGC *mark_gc;
-    mark_gc = sato_gc_new (style->base_gc[GTK_STATE_SELECTED], window);
-    gdk_draw_arc (window, mark_gc, TRUE, x + 5, y + 5, width - 10, height - 10, 0, 360 * 64);
-    g_object_unref (mark_gc);
+    gdk_draw_arc (window, style->text_gc[state_type], TRUE, x + 3, y + 3, width - 7, height - 7, 0, 360 * 64);
   }
-  gdk_draw_arc (window, gc, FALSE, x, y, width, height, 0, 360 * 64);
-
   g_object_unref (gc);
 }
 
@@ -512,59 +468,52 @@ sato_draw_extension (GtkStyle * style, GdkWindow * window,
 
   GdkGC *gc;
   gc = sato_gc_new (style->text_gc[state_type], window);
-  
-
 
   switch (gap_side)
   {
     case GTK_POS_TOP:     /* bottom tab */
+      gdk_draw_rectangle (window, style->bg_gc[state_type], TRUE, x + 1, y, width - 2, height - 2);
       sato_corner (window, gc, x, y + height, SATO_CORNER_BOTTOM_LEFT);
       sato_corner (window, gc, x + width, y + height, SATO_CORNER_BOTTOM_RIGHT);
-      y -= 5; height += 5;
+      /* adjust for line width */
+      x += 1; width -= 2; height -= 1;
+      gdk_draw_line (window, gc, x, y - 2, x, y + height - 2);
+      gdk_draw_line (window, gc, x + width, y - 2, x + width, y + height - 2);
+      gdk_draw_line (window, gc, x + 2, y + height, x + width - 2, y + height);
       break;
     case GTK_POS_BOTTOM: 	/* top tab */
+      gdk_draw_rectangle (window, style->bg_gc[state_type], TRUE, x + 2, y + 2, width - 4, height - 2);
       sato_corner (window, gc, x, y, SATO_CORNER_TOP_LEFT);
       sato_corner (window, gc, x + width, y, SATO_CORNER_TOP_RIGHT);
-      height += 5;
+      /* adjust for line width */
+      x += 1; y += 1; width -= 2; height += 1;
+      gdk_draw_line (window, gc, x, y + 2, x, y + height);
+      gdk_draw_line (window, gc, x + width, y + 2, x + width, y + height);
+      gdk_draw_line (window, gc, x + 2, y, x + width - 2, y);
       break;
     case GTK_POS_LEFT:	/* right tab */
+      gdk_draw_rectangle (window, style->bg_gc[state_type], TRUE, x, y + 2, width - 2, height - 4);
       sato_corner (window, gc, x + width, y, SATO_CORNER_TOP_RIGHT);
       sato_corner (window, gc, x + width, y + height, SATO_CORNER_BOTTOM_RIGHT);
-      x -= 5; width += 5;
+      /* adjust for line width */
+      x -= 2; y += 1; width += 1; height -= 2;
+      gdk_draw_line (window, gc, x, y, x + width - 2, y);
+      gdk_draw_line (window, gc, x, y + height, x + width - 2, y + height);
+      gdk_draw_line (window, gc, x + width, y + 2, x + width, y + height - 2);
       break;
     case GTK_POS_RIGHT:	/* left tab */
+      gdk_draw_rectangle (window, style->bg_gc[state_type], TRUE, x + 2, y + 1, width, height - 2);
       sato_corner (window, gc, x, y, SATO_CORNER_TOP_LEFT);
       sato_corner (window, gc, x, y + height, SATO_CORNER_BOTTOM_LEFT);
-      width += 5;
+      /* adjust for line width */
+      x += 1; y += 1; width += 1; height -= 2;
+      gdk_draw_line (window, gc, x + 2, y, x + width, y);
+      gdk_draw_line (window, gc, x + 2, y + height, x + width, y + height);
+      gdk_draw_line (window, gc, x, y + 2, x, y + height - 2);
       break;
   }
-  
-    if (state_type == GTK_STATE_NORMAL)
-    sato_gradient (style, window, state_type, x + 4, y + 4, width - 8,
-		   height - 8);
 
 
-
-  /* top and bottom */
-  y++;
-  height++;
-  if (gap_side != GTK_POS_TOP)
-    gdk_draw_line (window, gc, x + 3, y, x + width - 3, y);
-  if (gap_side != GTK_POS_BOTTOM)
-    gdk_draw_line (window, gc, x + 3, y + height - 3, x + width - 3,
-		   y + height - 3);
-  y--;
-  height--;
-
-  /* left and right */
-  x++;
-  width++;
-  if (gap_side != GTK_POS_LEFT)
-    gdk_draw_line (window, gc, x, y + 3, x, y + height - 3);
-  if (gap_side != GTK_POS_RIGHT)
-    gdk_draw_line (window, gc, x + width - 3, y + 3, x + width - 3,
-		   y + height - 3);
-
   g_object_unref (gc);
 
 }
@@ -575,10 +524,13 @@ sato_draw_vline (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
                  gint y1, gint y2, gint x)
 {
   GdkGC *line_gc;
-  line_gc = sato_gc_new (style->text_gc[state_type], window);
 
   if (DETAIL ("vscale") || DETAIL ("hscale"))
     return;
+
+  line_gc = sato_gc_new (style->text_gc[state_type], window);
+
+
   gdk_draw_line (window, line_gc, x, y1, x, y2);
 
   g_object_unref (line_gc);
@@ -591,10 +543,11 @@ sato_draw_hline (GtkStyle *style, GdkWindow *window,  GtkStateType state_type,
 {
   GdkGC *line_gc;
 
-  line_gc = sato_gc_new (style->text_gc[state_type], window);
-
   if (DETAIL ("vscale") || DETAIL ("hscale"))
     return;
+
+  line_gc = sato_gc_new (style->dark_gc[state_type], window);
+
   gdk_draw_line (window, line_gc, x1, y, x2, y);
 
   g_object_unref (line_gc);
@@ -617,18 +570,8 @@ sato_draw_style_class_init (GtkStyleClass * style_class)
   style_class->draw_hline = sato_draw_hline;
   style_class->draw_vline = sato_draw_vline;
   style_class->draw_focus = sato_draw_focus;
-/*
 
-  style_class->draw_arrow = draw_arrow;
-  style_class->draw_check = draw_check;
-  style_class->draw_option = draw_option;
-  style_class->draw_tab = draw_tab;
-  style_class->draw_shadow_gap = draw_shadow_gap;
-  style_class->draw_box_gap = draw_box_gap;
-  style_class->draw_extension = draw_extension;
-  style_class->draw_focus = draw_focus;
-  style_class->draw_slider = draw_slider;
-  style_class->draw_handle = draw_handle;
-  style_class->draw_layout = draw_layout;
-*/
 }
+
+
+#endif /* not ENABLE_CAIRO */
